@@ -94,4 +94,4 @@ def emit_queued(thread: Thread | None = None) -> None:
         try:
             cb(args)
         except Exception as e:  # pragma: no cover
-            raise EmitLoopError(cb=cb, args=args, exc=e) from e
+            raise EmitLoopError(exc=e) from e
