@@ -1,4 +1,5 @@
 import os
+from copy import copy
 from typing import List, cast
 from unittest.mock import Mock, call
 
@@ -312,7 +313,7 @@ def test_child_events():
     e_obj = E()
     root: EventedList[E] = EventedList(child_events=True)
     mock = Mock()
-    root.events.connect(mock)
+    root.events.all.connect(mock)
     root.append(e_obj)
     assert len(e_obj.test) == 1
     assert root == [e_obj]
@@ -346,12 +347,16 @@ def test_child_events_groups():
     e_obj = E()
     root: EventedList[E] = EventedList(child_events=True)
     mock = Mock()
-    root.events.connect(mock)
+    root.events.all.connect(mock)
     root.append(e_obj)
     assert root == [e_obj]
     e_obj.events.test2.emit("hi")
 
-    assert mock.call_count == 3
+    assert [c[0][0].signal.name for c in mock.call_args_list] == [
+        "inserting",
+        "inserted",
+        "child_event",
+    ]
 
     # when an object in the list owns an emitter group, then any emitter in that group
     # will also be detected, and child_event will emit (index, sub-emitter, args)
@@ -367,5 +372,12 @@ def test_child_events_groups():
 
     # note that we can get back to the actual object in the list using the .instance
     # attribute on signal instances.
-    assert e_obj.events.test2.instance.instance == e_obj
+    assert e_obj.events.test2.instance.all.instance == e_obj
     mock.assert_has_calls(expected)
+
+
+def test_copy_no_sync():
+    l1 = EventedList([1, 2, 3])
+    l2 = copy(l1)
+    l1.append(4)
+    assert len(l2) == 3
