@@ -11,6 +11,7 @@ import pytest
 
 import psygnal
 from psygnal import EmitLoopError, Signal, SignalInstance
+from psygnal._signal import ReemissionMode, ReemissionVal
 from psygnal._weak_callback import WeakCallback
 
 PY39 = sys.version_info[:2] == (3, 9)
@@ -124,14 +125,21 @@ def test_decorator():
     err = ValueError()
 
     @emitter.one_int.connect
-    def boom(v: int):
+    def boom(v: int) -> None:
         raise err
 
     @emitter.one_int.connect(check_nargs=False)
     def bad_cb(a, b, c): ...
 
-    with pytest.raises(EmitLoopError) as e:
-        emitter.one_int.emit(1)
+    import re
+
+    error_re = re.compile(
+        "signal 'tests.test_psygnal.Emitter.one_int'" f".*{re.escape(__file__)}",
+        re.DOTALL,
+    )
+    with pytest.raises(EmitLoopError, match=error_re) as e:
+        emitter.one_int.emit(42)
+
     assert e.value.__cause__ is err
     assert e.value.__context__ is err
 
@@ -393,7 +401,7 @@ def test_weakref(slot):
         "partial",
     ],
 )
-def test_group_weakref(slot) -> None:
+def test_group_weakref(slot: str) -> None:
     """Test that a connected method doesn't hold strong ref."""
     from psygnal import SignalGroup
 
@@ -809,14 +817,6 @@ def test_emit_loop_exceptions():
     mock1.assert_called_once_with(2)
 
 
-# def test_partial_weakref():
-#     """Test that a connected method doesn't hold strong ref."""
-
-#     obj = MyObj()
-#     cb = partial(obj.f_int_int, 1)
-#     assert _partial_weakref(cb) == _partial_weakref(cb)
-
-
 @pytest.mark.parametrize(
     "slot",
     [
@@ -979,9 +979,9 @@ def test_pickle():
 
 
 @pytest.mark.skipif(PY39 and WINDOWS and COMPILED, reason="fails")
-@pytest.mark.parametrize("recursion", ["immediate", "deferred"])
-def test_recursion_error(recursion: Literal["immediate", "deferred"]) -> None:
-    s = SignalInstance(recursion_mode=recursion)
+@pytest.mark.parametrize("strategy", [ReemissionMode.QUEUED, ReemissionMode.IMMEDIATE])
+def test_recursion_error(strategy: ReemissionMode) -> None:
+    s = SignalInstance(reemission=strategy)
 
     @s.connect
     def callback() -> None:
@@ -991,9 +991,11 @@ def test_recursion_error(recursion: Literal["immediate", "deferred"]) -> None:
         s.emit()
 
 
-@pytest.mark.parametrize("recursion", ["immediate", "deferred"])
-def test_callback_order(recursion: Literal["immediate", "deferred"]) -> None:
-    sig = SignalInstance((int,), recursion_mode=recursion)
+@pytest.mark.parametrize(
+    "strategy", [ReemissionMode.QUEUED, ReemissionMode.IMMEDIATE, ReemissionMode.LATEST]
+)
+def test_callback_order(strategy: ReemissionMode) -> None:
+    sig = SignalInstance((int,), reemission=strategy)
 
     a = []
 
@@ -1015,19 +1017,23 @@ def test_callback_order(recursion: Literal["immediate", "deferred"]) -> None:
     sig.connect(cb3)
     sig.emit(1)
 
-    if recursion == "immediate":
-        # nested emission events occur immediately,
-        # before proceeding to the next callback
+    if strategy == ReemissionMode.IMMEDIATE:
+        # nested emission events immediately trigger the next nested level
+        # before returning to process the remainder of the current loop
         assert a == [1, 2, 20, 3, 30, 300, 200, 10, 100]
-    elif recursion == "deferred":
+    elif strategy == ReemissionMode.LATEST:
+        # nested emission events immediately trigger the next nested level
+        # and never return to process the remainder of the current loop
+        assert a == [1, 2, 20, 3, 30, 300]
+    elif strategy == ReemissionMode.QUEUED:
         # all callbacks are called once before the next one is called
         assert a == [1, 10, 100, 2, 20, 200, 3, 30, 300]
 
 
-@pytest.mark.parametrize("recursion", ["immediate", "deferred"])
-def test_signal_order_suspend(recursion: Literal["immediate", "deferred"]) -> None:
+@pytest.mark.parametrize("strategy", [ReemissionMode.QUEUED, ReemissionMode.IMMEDIATE])
+def test_signal_order_suspend(strategy: ReemissionMode) -> None:
     """Test that signals are emitted in the order they were connected."""
-    sig = SignalInstance((int,), recursion_mode=recursion)
+    sig = SignalInstance((int,), reemission=strategy)
     mock1 = Mock()
     mock2 = Mock()
 
@@ -1053,9 +1059,9 @@ def test_signal_order_suspend(recursion: Literal["immediate", "deferred"]) -> No
     sig.emit(1)
 
     mock1.assert_has_calls([call(1), call(10), call(11), call(39)])
-    if recursion == "immediate":
+    if strategy == ReemissionMode.IMMEDIATE:
         mock2.assert_has_calls([call(11), call(39), call(10), call(1)])
-    else:
+    elif strategy == ReemissionMode.QUEUED:
         mock2.assert_has_calls([call(1), call(10), call(11), call(39)])
 
 
@@ -1110,3 +1116,17 @@ def test_emit_should_not_prevent_gc():
     something.changed.emit(obj)
     del obj
     assert len(object_instances) == 0
+
+
+@pytest.mark.parametrize("strategy", ReemissionMode._members())
+def test_emit_loop_error_message_construction(strategy: ReemissionVal) -> None:
+    sig = SignalInstance((int,), reemission=strategy)
+    sig.connect(lambda v: v == 1 and sig.emit(2))  # type: ignore
+    sig.connect(lambda v: v == 2 and sig.emit(0))  # type: ignore
+    sig.connect(lambda v: 1 / v)
+    with pytest.raises(EmitLoopError, match="While emitting signal") as e:
+        sig.emit(1)
+
+    if strategy == "queued":
+        # check that we show a useful message for confusign queued signals
+        assert "NOTE" in str(e.value)
