@@ -1,10 +1,10 @@
 """qtbot should work for testing!"""
+
 from threading import Thread, current_thread, main_thread
-from typing import TYPE_CHECKING, Any, Callable, Tuple
+from typing import TYPE_CHECKING, Any, Callable, Literal, Tuple
 from unittest.mock import Mock
 
 import pytest
-from typing_extensions import Literal
 
 from psygnal import Signal
 from psygnal._signal import _guess_qtsignal_signature
